function [lambda,factors,Sigma_ee,tol,count]=spce(X,n1,r0,r01,r02)
% Djogbenou, A. A. (2023) 
% Last modified: September 2023
% It gives us loadings and factors estimated from a 2-level factor panel
% model using the sequencial principal component

% Outputs:
% lambda:estimated loadings
% factors:estimated factors
% Sigma_ee: estimated variance for idiosyncratic errors
% tol: precision
% count: number of iterations

% Inputs:
% X: t*n matrix, 
% n1: Number of developed economies real activity variables
% r0: Number of global factor,
% r01,r02: Number of specific factors.

[t,n]=size(X);
n2=n-n1;
X1=X(:,1:n1);
X2=X(:,n1+1:end);

%First step Factor estimation
    [vectors,values,v] = svds(X*X'/(n*t),r0);       
    Ghat = sqrt(t)*vectors;
 

 %Initialization
  count=0;
  tol=1000;
  critere=1000;
         Lhat = X'*Ghat*inv(Ghat'*Ghat);
         Y=X-Ghat(:,1:r0)*Lhat';
 while tol > 0.000001  

         y1=Y(:,1:n1);
         y2=Y(:,1+n1:n); 
     %Step 2
          [Vectors1,Values1,V1] = svds(y1*y1'/(n1*t),r01); 
          Ghat1 = sqrt(t)*Vectors1;
          Lhat1 = y1'*Ghat1/t;
          Ehat1=X1-Ghat1*Lhat1';
          
          [Vectors2,Values2,V2] = svds(y2*y2'/(n2*t),r02);       
          Ghat2 = sqrt(t)*Vectors2;
          Lhat2 = y2'*Ghat2/t;
          Ehat2=X2-Ghat2*Lhat2';

     %Step 3
        Ehat=[Ehat1,Ehat2];
        [Vectors,Values,V] = svds(Ehat*Ehat'/(n*t),r0);       
        Ghat = sqrt(t)*Vectors;
        Lhat = Ehat'*Ghat/t;
        Y=X-Ghat(:,1:r0)*Lhat';
       
     critere0=mean(mean((Ehat-Ghat*Lhat').^2));
     count=count+1;    
     tol=abs(critere-critere0);
     critere=critere0;     

     Sigma_ee=diag(mean((Ehat-Ghat*Lhat').^2,1));
     lambda=[Lhat,[Lhat1; zeros(n2,1)],[zeros(n1,1);Lhat2]];
     factors=[Ghat,Ghat1,Ghat2];
  end 
 

